#'Creating the distance Matrix of a subsampled imgage
#'
#'The function \code{GWBarycenterSample} takes an image in matrix-form, subsamples the poits with mass (i.e. that are not equal to zero) and computes the
#'corresponding distance matrix of these points.
#'@param image The image which should be subsampled in matrix form.
#'@param nop The number of pixels (the images are assumed to be quadratic).
#'@param sample.size The number of points that shall be subsampled.
#'@return The distance matrix of the subsampled points (the image is assumed to be in [0,1]²).
#'
#'
#'@export
GWBarycenterSample<-function(image,nop, sample.size){
  # the correspondin indices of the non zero values
  tmp = which(image>0, arr.ind = T)
  vert.x <- seq(0,1,(1-0)/(nop-1))
  vert.y <- seq(0,1,(1-0)/(nop-1))

  sample.indices = sample(1:dim(tmp)[1], sample.size, replace = T)

  vert.x.sample <- vert.x[tmp[sample.indices,1]]
  vert.y.sample <- vert.x[tmp[sample.indices,2]]
  # returning the distance matrices
  grid <- as.matrix(dist(cbind(vert.x.sample ,vert.y.sample), diag = TRUE, upper = TRUE))
  return(grid)
}
