
distance.matrix.grid <- function(num.x,left.x,right.x,num.y,left.y,right.y){

  #' @title Computing a distance matrix of a grid
  #' @description The function \code{distance.matrix.grid} computes
  #' a distance matrix for a grid of x x y lattice points on the set
  #' [left.x,right.x] x [left.y,right.y].
  #' @param  num.x: The number of lattice points in x-direction
  #' @param  left.x: The left border for lattice points in x-direction
  #' @param  right.x: The right border for lattice points in x-direction
  #' @param  num.y: The number of lattice points in y-direction
  #' @param  left.y: The left border for lattice points in y-direction
  #' @param  right.y: The right border for lattice points in y-direction
  #' @return The distance matrix of the grid

  vert.x <- seq(left.x,right.x,(right.x-left.x)/(num.x))
  vert.y <- seq(left.y,right.y,(right.y-left.y)/(num.y))

  if (num.x >1){
    vert.x <- (vert.x[1:(num.x)]+vert.x[2:(num.x+1)])/2
  } else {
    vert.x = (left.x +right.x)/2
  }

  if (num.y >1){
    vert.y <- (vert.y[1:(num.y)]+vert.y[2:(num.y+1)])/2
  } else {
    vert.y = (left.y +right.y)/2
  }


  u <- expand.grid(vert.x,vert.y)

  grid <- as.matrix(dist(u, diag = TRUE, upper = TRUE))

  return(list("dist"=grid, "u" = u))
}
