% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{PerformGWSinkhorn}
\alias{PerformGWSinkhorn}
\title{Computing the entropic Gromov-Wasserstein distance}
\usage{
PerformGWSinkhorn(d1, d2, mu1, mu2, epsilon, options)
}
\arguments{
\item{d1:}{The distance matrix of all points in X.}

\item{d2:}{The distance matrix of all points in Y.}

\item{mu1:}{The weights of the points in X as column vector.}

\item{mu2:}{The weights of the points in Y as column vector.}

\item{epsilon:}{A numeric parameter of the entropic Gromov-Wasserstein distance.}

\item{options:}{A named list containing several options concerning the methods of computation.
Possible options are:
\describe{
\item{"niter"}{The max. number of projected gradient descent steps (default = 100).}
\item{"niter_sinkhorn"}{The max. number of steps in order to calculate a projection (default =400).}
\item{"gw_loss"}{A string marking the choice of the used loss. Possibilities are the well known L2 loss "L2" (default) and the Kullback-Leibler divergence "KL".}
\item{"tol_gw"}{A numeric value in order to stop if the progress in the projection steps is smaller than this value (default = 10^-6).}
\item{"safe"}{A binary option in order to increase the stability of the calcukations (default =0).}
\item{"gamma_init"}{A possible start coupling (as matrix) in order to speed up the calculations.}
\item{"tol_sinkhorn"}{A numeric value in order to stop if the progress in the calculation of a projection steps is smaller than this value  (defalut =10^-10).}
\item{"progress"}{A binary option to enable/disable a progressbar (default = 0, choose 1 to enable ).}
\item{"control"}{A binary option to control the progress of the calculations (default = 1). Setting this to 0 will speed up the calculations, but prevent them from being cancelled via the R console.}
}}
}
\value{
A List containing a vector with the values of the projection steps while calculating the entopic Gromov-Wasserstein distance  between the 2  lattices X and Y ($distance) and the final coupling ($gamma).
}
\description{
The function \code{PerformGWSinkhorn} computes the
the entropic Gromov-Wasserstein distance between two discrete metric measure spaces X and Y via
projected gradient descent (cf. “Gromov-Wasserstein Averaging of Kernel and Distance Matrices.” by Peyré,
Gabriel, Marco Cuturi, and Justin Solomon in ICML 2016, 2016.).
}
\examples{
 We consider the well known example of the n-simplex (cf. "Gromov–Wasserstein Distances and the Metric Approach to Object Matching", by F. Mémoli in  F. Found Comput Math, 2011). This is a
 metric measure space consiting of n points, which have mass 1/n and distance 1 to all other points.
 For these sturutures some explicit expressoins and many bounds for the Gromov-Wasserstein distance are known.


 m <- 4
 y.distmat<- matrix(0, nrow = 1, ncol =1)
 x.distmat<- matrix(1, nrow = m, ncol =m)-diag(1,nrow=m, ncol =m)
 eps <- 0.001
 y.weight <- 1
 x.weight <- rep(1/m,m)
 options = list("niter"=40,"niter_sinkhorn"= 100, tol_gw = 1e-6, "tol_sinkhorn"= 1e-6, "progress" = 1)

 PerformGWSinkhorn(x.distmat, y.distmat, x.weight, y.weight, eps, options)
}
