/* This file is a matlab mex function for computing the transportation cost
 * between two vectors given a cost matrix.
 *
 * It was written by Antoine Rolet (2014) and mainly consists of a wrapper
 * of the code written by Nicolas Bonneel available on this page
 *          http://people.seas.harvard.edu/~nbonneel/FastTransport/
 *
 * Please give relevant credit to the original author (Nicolas Bonneel) if
 * you use this code for a publication.
 *
 */
#include <RcppArmadillo.h>
#include <stdexcept>
#include <vector>
#include <iostream>
#include "network_simplex_simple.h"
#define   DEBUG  0

//[[Rcpp::depends(RcppArmadillo)]]

using namespace lemon;
typedef unsigned int node_id_type;


void NetworkSimplexfast( arma::vec& mu,arma::vec& nu,arma::mat& C, arma::mat& gamma) {
// This function uses the network simplex to solve a transport problem and stores the result
// in gamma
  int n, m, i, n1, n2,cur;
  double *X, *Y, *D, *demd, *optimal, max,max_iter;
  int dimx[2], dimy[2], dimd[2], dimw[2];

  typedef FullBipartiteDigraph Digraph;
  DIGRAPH_TYPEDEFS(FullBipartiteDigraph);




  // Get the input arguments
  // wird bei Gelegenheit geändert D: maybe also change to constant int*
  dimx[0]= mu.n_rows;
  dimx[1] = mu.n_cols;

  dimy [0]=nu.n_rows;
  dimy [1]=nu.n_cols;

  dimd[0]=C.n_rows;
  dimd[1]=C.n_cols;


  if(DEBUG){
    printf("dimx[0] is given as %i\n", dimx[0]);
    printf("dimx[1] is given as %i\n", dimx[1]);
    printf("dimy[0] is given as %i\n", dimy[0]);
    printf("dimy[1] is given as %i\n", dimy[1]);
    printf("dimd[0] is given as %i\n", dimd[0]);
    printf("dimd[1] is given as %i\n", dimd[1]);
  }

  X = mu.memptr();
  Y = nu.memptr();
  D = C.memptr();

  if(DEBUG){
    printf("X points to %f\n", *X);
    printf("X points to %f\n", *(X+1));

  }
// Always 3 input arguments
  max_iter=-1;
  if(DEBUG){
    printf("Setting the input was correct.\n");
  }
  // Check dimensions

  if (dimx[1] != 1 || dimy[1] != 1){
    throw std::runtime_error("First two arguments should be column vectors.");
  }
  n1 = dimd[0];
  n2 = dimd[1];

  if(DEBUG){
    printf("decision value of the first if is %i\n", dimx[1] != 1||dimy[1] != 1);
    printf("n1 is given as %i\n", n1);
    printf("n2 is given as %i\n", n2);
  }

  if (dimx[0] != n1 || dimy[0] != n2){
    throw std::runtime_error("Ground metric matrix dimensions should be the length of the two first arguments");

  }
  // Get the number of non zero coordinates for r and c
    n=0;
    for (node_id_type i=0; i<n1; i++) {
        double val=*(X+i);
        if (val>0) {
            n++;
        }
    }
    m=0;
    for (node_id_type i=0; i<n2; i++) {
        double val=*(Y+i);
        if (val>0) {
            m++;
        }
    }


    // Define the graph
    if(DEBUG){
      printf("n is given as %i\n",n);
      printf("m is given as %i\n", m);
    }
    std::vector<int> indI(n), indJ(m);
    std::vector<double> weights1(n), weights2(m);
    Digraph di(n, m);
    NetworkSimplexSimple<Digraph,double,double, node_id_type> net(di, true, n+m, n*m,max_iter);

    // Set supply and demand, don't account for 0 values (faster)

    max=0;
    cur=0;
    for (node_id_type i=0; i<n1; i++) {
        double val=*(X+i);
        if (val>0) {
            weights1[ di.nodeFromId(cur) ] = val;
            max+=val;
            indI[cur++]=i;
        }
    }

    // Demand is actually negative supply...

    max=0;
    cur=0;
    for (node_id_type i=0; i<n2; i++) {
        double val=*(Y+i);
        if (val>0) {
            weights2[ di.nodeFromId(cur) ] = -val;
            indJ[cur++]=i;

            max-=val;
        }
    }


    net.supplyMap(&weights1[0], n, &weights2[0], m);

    // Set the cost of each edge
    max=0;
    for (node_id_type i=0; i<n; i++) {
        for (node_id_type j=0; j<m; j++) {
            double val=*(D+indI[i]+indJ[j]*n1);
            net.setCost(di.arcFromId(i*m+j), val);
            if (val>max) {
                max=val;
            }
        }
    }


    // Solve the problem with the network simplex algorithm

    int ret=net.run();
    if (ret!=(int)net.OPTIMAL) {
        if (ret==(int)net.INFEASIBLE) {
          throw std::runtime_error("Infeasible problem");
        }
        if (ret==(int)net.UNBOUNDED) {
          throw std::runtime_error("Unbounded problem");
        }
    }

    // Define the output arguments

	  arma::mat res(1,1);
	  demd = res.memptr();


		optimal = gamma.memptr();
		if(DEBUG){
		  printf("Definig the output was correct\n");
		}

    // Set the output values

    if (1) {
      for (node_id_type i=0; i<n; i++) {
        for (node_id_type j=0; j<m; j++)
        {
          *(optimal+indI[i]+indJ[j]*n1) = net.flow(di.arcFromId(i*m+j));
        }
      }
    }

    *demd=net.totalCost();
    // später vllt auch die Kosten !
    return;
}
