#include <RcppArmadillo.h>
//[[Rcpp::depends(RcppArmadillo)]]
#include <R.h>
#include <Rdefines.h>
using namespace Rcpp;

// Using a clever way to set possible default values (similar to the matlab code of Peyré).

int getoptions_int(List& options,  const char* name, int v){

// getoptions - retrieve options parameter
//
//   v = getoptions(options, 'entry', v0);
//      is equivalent to the code:
//   if isfield(options, 'entry')
//       v = options.entry;
//   else
//       v = v0;
//   end
//
  int erg;
  if (options.containsElementNamed(name)){
   erg =  options[name];
  }
  else{
    erg = v;
  }
  return(erg);
}


double getoptions_double(List& options,  const char* name, double v){

  // getoptions - retrieve options parameter
  //
  //   v = getoptions(options, 'entry', v0);
  //      is equivalent to the code:
  //   if isfield(options, 'entry')
  //       v = options.entry;
  //   else
  //       v = v0;
  //   end
  //
  double erg;
  if (options.containsElementNamed(name)){
    erg =  options[name];
  }
  else{
    erg = v;
  }
  return(erg);
}


arma::vec getoptions_vec(List& options,  const char* name, arma::vec v){

  // getoptions - retrieve options parameter
  // Same function as above, but now we set Armadillo vectors.
  arma::vec erg;
  if (options.containsElementNamed(name)){
    erg =  Rcpp::as<arma::vec>(options[name]);
  }
  else{
    erg = v;
  }
  return(erg);
}



arma::mat getoptions_mat(List& options,  const char* name, arma::mat v){

  // getoptions - retrieve options parameter
  // Same function as above, but now we set Armadillo matrices.
  arma::mat erg;
  if (options.containsElementNamed(name)){
    erg =  Rcpp::as<arma::mat>(options[name]);
  }
  else{
    erg = v;
  }
  return(erg);
}



std::string getoptions_string(List& options,  const char* name, const char* v){

  // getoptions - retrieve options parameter
  // Same function as above, but now we set strings.
  std::string erg;
  if (options.containsElementNamed(name)){
    erg =  Rcpp::as<std::string>(options[name]);
  }
  else{
    erg = v;
  }
  return(erg);
}


SEXP setInt(int a) {

  SEXP myint;
  int *p_myint;


  PROTECT(myint = NEW_INTEGER(1)); // Allocating storage space
  p_myint = INTEGER_POINTER(myint); // ponit to SEXP object

  p_myint[0] = a;

  UNPROTECT(1);
  return myint;
}


SEXP setDouble(double a) {
  SEXP myint;
  double *p_myint;


  PROTECT(myint = NEW_NUMERIC(1)); // Allocating storage space
  p_myint = NUMERIC_POINTER(myint); // ponit to SEXP object

  p_myint[0] = a;

  UNPROTECT(1);
  return myint;
}




