#include <RcppArmadillo.h>
#include <stdexcept>
#include "getoptions.h"
#include "supportfct.h"
#include "Networksimplex.h"
#include "perform_log_sinkhorn.h"
#define   DEBUG  0
//[[Rcpp::depends(RcppArmadillo)]]

using namespace Rcpp;

void PerformGWSinkhornOTVersion(arma::mat& d1,arma::mat& d2,arma::vec& mu1,arma::vec& mu2, List& options, List& results){



  // Choose the corresponding loss
  std::string loss = getoptions_string(options, "gw_loss", "L2");
  unsigned int chosen_loss;
  if(!loss.compare("L2")){
    chosen_loss = 0;
  }
  else{
    chosen_loss=1;
  }
  // Create an arry of function pointers
  typedef arma::mat (*fptr_t)(arma::mat);
  fptr_t f_one[2] = {f_one_L2, f_one_KL};
  fptr_t f_two[2] = {f_two_L2,f_two_KL};
  fptr_t h_one[2]=  {h_one_L2, h_one_KL};
  fptr_t h_two[2] = {h_two_L2, h_two_KL};


  options["null"] = 0;
  int niter = getoptions_int(options, "niter", 100);
  int niter_sinkhorn = getoptions_int(options, "niter_sinkhorn", 400);
  double tol_gw = getoptions_double(options, "tol_gw", 1e-6);
  int safe = getoptions_int(options, "safe", 0);
  int progress = getoptions_int(options, "progress",0);
  int control = getoptions_int(options, "control", 1);
  int log_domain = getoptions_int(options, "log_domain", 0);

  double GWcst = dotp( f_one[chosen_loss](d1)*mu1,mu1) + dotp( f_two[chosen_loss](d2)*mu2,mu2);
  NumericVector GW;

  // initial match
  arma::mat gamma = getoptions_mat(options, "gamma_init", mu1*trans(mu2));
  // setting options for perform_sinkhorn
  double tol= getoptions_double(options, "tol_sinkhorn", 1e-10);

  // Preparing the possible log domain appraoch
  List res_skl;


  if(DEBUG){
    printf("niter is set as:%i \n", niter);
    printf("niter_sinkhorn is set as:%i \n", niter_sinkhorn);
    printf("The chosen loss is:%i \n", chosen_loss);
    printf("tol_gw is set as:%f \n", tol_gw);
    printf("safe is set as:%i \n", safe);
    printf("tol_sinkhorn is set as:%f \n", tol);
    printf("safe is set as:%i \n", safe);
    printf("control is set as:%i \n", control);
    printf("gamma_init is given as:\n");
    showMatrix(gamma);

  }



  if(DEBUG){
    printf("d1 is given as:\n");
    showMatrix(d1);
    printf("d2 is given as:\n");
    showMatrix(d2);
    printf("mu1 is given as:\n");
    showVector(mu1);
    printf("mu2 is given as:\n");
    showVector(mu2);

  }
  arma::mat gamma_res;

  for (int i=0; i<=niter-1; i++){

    arma::mat C = - h_one[chosen_loss](d1)*gamma*h_two[chosen_loss](trans(d2));
    arma::mat C_zero;

    C_zero = C-vectorise(C).min();
    // run the Network Simplex
    if(DEBUG){
      printf("C_zero is given as:\n");
      showMatrix(C_zero);

    }

    if(log_domain ==0){
      // gamma is the Matrix where the result is stored
      NetworkSimplexfast(  mu1, mu2, C_zero, gamma);
    }
    else{

      double epsilon = getoptions_double(options, "epsilon", 1e-6);
      PerformSinkhornLog(C_zero, epsilon,  mu1, mu2,  options,res_skl);
      gamma = as<arma::mat>(res_skl["gamma"]);
    }

    // Checking the results of perform_sinkhorn
    if(DEBUG){
      printf("for iteration %d \n", i);
      printf("The dimension of C is %ix%i \n", size(C,0), size(C,1));
      showMatrix(C);
      printf("The interim result for gamma is:\n");
      showMatrix(gamma);}

      if(DEBUG){
      // Checking all the results used for calculating GW
      //printf("GW-interim result for Entropy(gamma):%f \n", Entropy(gamma));
      printf("GW-interim result for  dotp( C,gamma )%f and i given as:%d \n", dotp( C,gamma ),i);
      //printf("GW-interim result for GWcst  :%f \n", GWcst);
    }

    GW.push_back(GWcst + dotp( C,gamma ));

    //printf("GW-interim result for GW[%i]  :%.10f \n",i, GW[i]);



    if ((i>1) && (tol_gw>0) && ( (min(as<NumericVector>(GW[(seq_len(i))-1])) - GW(i))/(std::abs(GW(0)))<tol_gw )){
      // stop of not enough progress

      break;
    }
    if (std::isnan(gamma(0))){
      throw std::runtime_error("Computation blew up, epsilon too small");
    }
  }

  results["distance"]=GW;
  results["gamma"]=gamma;
  return;

 }
