#include <RcppArmadillo.h>
#include "getoptions.h"
#include <stdexcept>
#include  "supportfct.h"
#define DEBUG 0
//[[Rcpp::depends(RcppArmadillo)]]
// [[Rcpp::depends(RcppProgress)]]
#include <progress.hpp>

using namespace Rcpp;

void  PerformSinkhornLog(arma::mat& c, double& epsilon, arma::mat& p,arma::mat& q, List& options, List& results){

// perform_sinkhorn_log - perform Sinkhorn over the log-domain.
//
//   [gamma,r,s,err] = perform_sinkhorn_log(p,q, c,epsilon, options);
//
//   Perform Sinkhorn algorithm partially over the log-domain (i.e. with
                                                              //   stabilization) to solve
//       min_{gamma} <c,gamma> + epsilon*sum( gamma(:).*log(gamma(:)) )
//   subject to
//       gamma*1=p
//       gamma'*1=q
//
//   (r,s) are dual variables such that
//       gamma = exp( -(c-r*1'-1*s')/epsilon );
//
//   err(i) = norm( gamma*1-p,1 ) for the current iterate i.
//
//   option.tol is a tolerance, forcing exit if err(i)<tol.
//
//   options.niter is the total number of iterations.
//   options.niter_sink is the number of inner Sinkhorn stabilization loop.
//      - Setting niter_sink=niter corresponds to the usual Sinkhorn, which is
//       unstable for small epsilon.
//      - Setting niter_sink=1 corresponds to doing all the iterates over
//       the log domain, which is slow.
//
//   Set options.verb=0 if you do not want to display a progressbar.
//
//   Copyright (c) 2015 Gabriel Peyre

  options["null"] = 0;
  int niter = getoptions_int(options, "niter", 1000);
  int niter_sink = getoptions_int(options, "niter_sink", 50);
  double tol = getoptions_double(options, "tol", 1e-6);

  int control = getoptions_int(options, "control", 1);
  int Np = c.n_cols;
  int Nq = c.n_rows;

  arma::mat up(Np,1,arma::fill::ones);
  arma::mat uq(Nq,1,arma::fill::ones );


  arma::mat r = getoptions_mat(results, "r_init", arma::zeros<arma::mat>(Np,1));
  arma::mat s = getoptions_mat(results, "s_init", arma::zeros<arma::mat>(Np,1));

  NumericVector err;
  int j;
  arma::mat gamma;
  arma::mat b;
  arma::mat a;

  for (int i=0; i <=(ceil(niter/niter_sink)-1); i++){
    gamma = exp( -(c-r*uq.t()-up*s.t())/epsilon );
    b =uq;
    // sinkhorn
    for (j=0; j<niter_sink;j++){
       a = p / (gamma*b);
       b = q / (gamma.t()*a);
      if (tol>0){
        err.push_back(norm( a%(gamma*b)-p, 1 ));
        if (err(j)<tol){
          // early stop
          break;
        }
      }
    }

    r = r + epsilon*log(a);
    s = s + epsilon*log(b);
    if (err.size()>0 && err(i)<tol){
    // early stop
      break;
    }
  }
  gamma =(exp(-(c-r*uq.t()-up*s.t())/epsilon ));

  results["r_init"]=r;
  results["s_init"]=s;
  results["gamma"]=gamma;

  if(1){
    //printf("gamma is given as:\n");
    //showMatrix(as<arma::mat>(results["gamma"]));
    double sum = arma::sum(arma::vectorise(as<arma::mat>(results["gamma"])));
    printf("The mean of gamma is given as: %f\n", sum );
  }

  return;
}
