#include <RcppArmadillo.h>
#include "getoptions.h"
#include <stdexcept>
#define DEBUG 0
//[[Rcpp::depends(RcppArmadillo)]]
// [[Rcpp::depends(RcppProgress)]]
#include <progress.hpp>

using namespace Rcpp;

Rcpp::List perform_sinkhorn(arma::mat& C,double& epsilon,arma::vec& mu, arma::vec& nu, Rcpp::List& options){


 if(DEBUG){
  printf("sinkhorn correctly called \n");
 }

  Progress progesscontrol(0, false); // in any case, we need to build an instance
  options["null"] = 0;
  int niter = getoptions_int(options, "niter", 100);
  double tol =  getoptions_double(options, "tol", 1e-10);
  arma::vec a = getoptions_vec(options, "a_init", arma::ones<arma::vec>(size(C,0),1));

  int safe = getoptions_int(options, "safe", 0);
  int control = getoptions_int(options, "control", 1);
  // gibbs kernel
   arma::mat K = exp(-C/epsilon);

  if (safe){
    // change elements of K smaler than 10^-200 to 10^-200
    K.elem(arma::find(K < pow(10,-200)) ).fill(pow(10, -200)); // Safe
  }

  arma::mat Err(1,2,arma::fill::zeros);
  arma::vec b   = arma::vec();

  for (int i=0; i<=(niter-1); i++){
    b = nu / (K.t()*a);
    Err = reshape(Err, i+1, 2);
    if (tol>0){
    Err(i,0) = norm(a%(K*b)-mu, 1);
    }
    a = mu / (K*b);
    if ( tol>0){
      Err(i,1) = norm(b%(K.t()*a)-nu, 1);
    }
    if (max(Err.row(i))<tol){
      break;
    }
    // option to cancel the calculations from R
    if(control){
      if(Progress::check_abort()){
        throw std::runtime_error("Calculations have been manually cancelled.");
      }
    }
  }
  if(DEBUG){
    printf("sinkhorn correctly terminated \n");
  }
  arma::mat gamma = arma::diagmat(a)*K*arma::diagmat(b);
  return(Rcpp::List::create(Named("gamma")=gamma,Named("a")=a,Named("b")= b, Named("Err")=Err));

}

